@@ -10,6 +10,7 @@ __all__ = (
     "Unauthorized",
     "SubscriptionDidNotSucceed",
     "PersistedQueryNotFound",
+    "SubscribeError",
 )
 
 
@@ -47,3 +48,7 @@ class SubscriptionDidNotSucceed(SocketError):
 
 class PersistedQueryNotFound(PnWKitException):
     ...
+
+
+class SubscribeError(PnWKitException):
+    ...
